#include <math.h> 
#include <stdio.h> 
#include "../ansi_lib_loc/io_mat.h"
#include "../ansi_lib_loc/A_comp_dbl_storeflt.h"
#include "../ansi_lib_loc/allocate.h"
#include "../ansi_lib_loc/randlib.h"


#ifndef PI
#define PI        3.14159265358979323846
#endif

#define NOISY

int VERBOSE=1;


void main(int argc, char **argv)
{
  /*FILE *fp;*/
  MATFile *fout,*fin;
  float **Y,**image;
  int N,M,i,j;
  struct column sp_col;
  struct sino_info sino;
  struct geom_info geom;
  int proj_ind_1D,t;
  double **profiles;
  int resol;

  float **tmp;

  srandom2(1);
  writeseed();

  tmp=(float**)multialloc(sizeof(float),2,1,1);
  
  sp_col.A_cor.mode='t';



 if (argc!=3)
  {
   fprintf(stderr,"\nUsage: forward in_file out_file\n\n");
   exit(-1);
}

  /* Read input .mat file */
  fin = matOpen(argv[1], "r");
  if (fin == NULL) {
    fprintf(stderr,"Cannot read MatLab file %s \n",argv[1]);
    exit(-1);
  }

  /* read counts matrix into Y */
  /* Routine  read_mat_float_image is in ansi_lib/io_mat.c */
  image = read_mat_float_image(&geom.n_x,&geom.n_y,"img",fin);
  if (image == NULL) 
    {
     image = read_mat_float_image(&geom.n_x,&geom.n_y,"X0",fin);
     if (image == NULL) 
       {
         fprintf(stderr,"Cannot read img or X0 matrix \n");
         exit(-1);
        }
    }
  /* Read n_theta, n_t, delta_theta, delta_t, theta_0, t_0,  dec_a=1, dec_t=1*/
  /* Routine in ansi_lib/Acomp.c */
  /*read_mat_sino_info(fin, &sino);*/
  matClose(fin);

 

  /* THIS IS WHERE YOU NEED TO SET YOUR PARAMETERS FOR THE
     FORWARD PROJECTION */
  sino.delta_t=0.7812; /* in cm */
  sino.n_t=256; 
  sino.n_theta=128;
  sino.theta_0=0.0; 
  resol=256;
  
 
  sino.delta_theta=PI/sino.n_theta;
  sino.t_0=-1.0*((double)(sino.n_t - 1))/2.0 * sino.delta_t;

  /* Set up geometry of reconstruction */
  geom.dia = sino.n_t*sino.delta_t;
  geom.n_y = resol; 
  geom.n_x = resol;
  geom.delta = geom.dia/(double)resol;
  geom.x_0 = (-geom.dia+geom.delta)/2.0; /* set origin */
  geom.y_0 = (-geom.dia+geom.delta)/2.0;

 
  /* routine pix_prof_comp is in Acomp.c */ 
  pix_prof_comp(&sino, &geom, &profiles);
  
  /* Allocate memory for Acomp  - row */
  sp_col.val = (double *) get_spc(sino.n_t*sino.n_theta, sizeof(double));
  sp_col.row = (int *) get_spc(sino.n_t*sino.n_theta, sizeof(int));
  
  N = geom.n_y;
  M = geom.n_x;


  printf("sino.n_t=%d  sino.n_theta=%d  sino.t_0=%f sino.theta_0=%f  sino_delta_t=%f  sino.delta_theta=%f\n",sino.n_t,sino.n_theta,sino.t_0, sino.theta_0, sino.delta_t, sino.delta_theta);


  

  /********* Allocate space for Sinogram ************/
  Y = (float **)get_img(sino.n_t, sino.n_theta, sizeof(float));
  for(i=0; i<sino.n_t*sino.n_theta; i++)
    Y[0][i] = 0.0;
      

  /* forward projection */
  for(i=0; i<N; i++) 
    for(j=0; j<M; j++) 
      {
	A_comp(i,j,&sino, &geom, profiles,&sp_col,0);
	for(t=0; t<sp_col.n_row; t++) 
	  {
	    proj_ind_1D = sp_col.row[t];
	    Y[0][proj_ind_1D] += (image[i][j]*(float)sp_col.val[t]);
	  }
      }
  
  /* UNCOMMENT FOR TRANSMISSION CASE */
  /*for(i=0; i<sino.n_t*sino.n_theta; i++)
    Y[0][i] = exp(-Y[0][i]);*/

#ifdef NOISY
  /* add noise i.e. replace projection values by Poisson RV */
  /* with mean=proj.val */
  for (i=0;i<sino.n_theta*sino.n_t;i++)
    Y[0][i] = (float)poidev((double)Y[0][i]);
  
#endif


  fout = matOpen(argv[2], "w");
  if (fout == NULL) 
    {
      fprintf(stderr,"Cannot write to MatLab file %s\n",argv[2]);
      exit(-1);
    }


  write_mat_float_image(Y,sino.n_theta,sino.n_t,"counts",fout);

  tmp[0][0]=(float)sino.delta_t;
  write_mat_float_image(tmp,1,1,"delta_t",fout);

  tmp[0][0]=(float)sino.delta_theta;
  write_mat_float_image(tmp,1,1,"delta_theta",fout);

  tmp[0][0]=(float)sino.n_t;
  write_mat_float_image(tmp,1,1,"n_t",fout);

  tmp[0][0]=(float)sino.n_theta;
  write_mat_float_image(tmp,1,1,"n_theta",fout);

  tmp[0][0]=0.0;
  write_mat_float_image(tmp,1,1,"scatter",fout);

  tmp[0][0]=(float)sino.t_0;
  write_mat_float_image(tmp,1,1,"t_0",fout);

  tmp[0][0]=(float)sino.theta_0;
  write_mat_float_image(tmp,1,1,"theta_0",fout);
 



  matClose(fout);

  free((void *)sp_col.val);
  free((void *)sp_col.row);
  free_img((void **)profiles);
  free_img((void **)image);
  free_img((void **)Y);


}




